#!ruby
#**********************************************************************
# def_rating_files.rb
# 競技かるたレーティングシステム(Ruby版)
# クラス定義 RatingFiles      ; ファイル／ディレクトリ処理
#**********************************************************************

require 'nkf'
require 'fileutils'

DEBUG_MODE = false		# true:デバッグ中、false:本番

# レーティング集計ツールのベースディレクトリ
RATINGDIR = 'C:/Karuta/rating2'

FILE_ERROR         = "error.txt"
FILE_ERROR_BAK     = "error.bak"
FILE_UPDATEMEMO    = "updatememo.txt"
FILE_GAME          = "game.txt"
FILE_MEIBO         = "meibomaster.txt"
FILE_MEIBO_BAK     = "meibomaster.o"
FILE_MEIBO_SAV     = "meibomaster.sav"
FILE_MERGE         = "merge.txt"
FILE_PLAYERS       = "players.txt"
FILE_RESULT        = "result.txt"
FILE_RESULT1       = "result1.txt"
FILE_RESULT2       = "result2.txt"
FILE_SAMENAME      = "samename.txt"
FILE_SAMENAME2     = "samename2.txt"
FILE_SYUUKEI       = "syuukei.txt"
FILE_CSVLIST       = "csvlist.txt"
FILE_RANKFILE      = "rankfile.txt"
FILE_RANKING       = "ranking.txt"
FILE_RANKHTML      = "rank.htm"
FILE_UNRANK        = "meibo_unranking.txt"


# correctpoint.rb用

FILE_PT_MEIBO      = "meibomaster_new.txt"
FILE_PT_PLAYER4TH  = "correctpt_player4th.txt"
FILE_PT_HOSEI      = "correctpt_result.txt"
FILE_PT_HOSEILOG   = "correctpt_result_log.txt"
FILE_PT_GREP_GAME  = "correctpt_grep_game.txt"


#======================================================================
# RatingFiles       ; ファイル／ディレクトリ処理
#======================================================================
class RatingFiles

# 初期化を行う
	def initialize(year = "")
		@@year       = year
		@@tooldir    = RATINGDIR + '/rbtool'
		@@yeardir    = RATINGDIR + '/' + @@year
		@@gamedir    = @@tooldir + '/game'
		@@meibodir   = @@tooldir + '/meibo'
		@@rankdir    = @@tooldir + '/rank'
		@@ydbdir     = @@tooldir + '/ydb'
		@@ydbfile    = @@ydbdir + '/' + year + '.ydb'
		@@ydbfile_ly = @@ydbdir + '/' + (year.to_i - 1).to_s + '.ydb'

		# 大会結果CSVファイルのリスト
		@@csv = []
	end

#######################################################################
# レーティング集計ツールのディレクトリに関する処理

# メインのツールディレクトリを得る
	def rating_tooldir
		return @@tooldir
	end


#######################################################################
# 大会結果CSVファイルに関する処理

# 集計対象西暦年の大会結果CSVファイルのリストを作成する
	def make_csvlist
		@@csvlist = []
		csvbuf = []
		i = 0
		csvbuf = Dir.glob("#{@@yeardir}/#{@@year}*.csv")
		csvbuf.each { |t|
			t.chomp!
			taikaicsv = t[(@@yeardir.length + 1),t.length]
			if taikaicsv.slice(0,8) =~ /^[0-9]+$/
				@@csvlist[i] = taikaicsv
				i += 1
			end
		}
		@@csvlist.sort!

		Dir.chdir(@@tooldir)
		File.open(FILE_CSVLIST, "w") { |f| f.puts @@csvlist }

		return @@csvlist
	end

# 大会結果CSVファイルのファイル名から大会開催日を得る(yyyy/m/d)
	def date_ymd_taikaicsv(filename = "")
		date_y = filename.slice(0,4).to_i
		date_m = filename.slice(4,2).to_i
		date_d = filename.slice(6,2).to_i
		date = sprintf("%d/%d/%d", date_y, date_m, date_d)
		return date
	end

# 大会結果CSVファイルのファイル名から大会開催日を得る(m/d)
	def date_md_taikaicsv(filename = "")
		date_m = filename.slice(4,2).to_i
		date_d = filename.slice(6,2).to_i
		date = sprintf("%d/%d", date_m, date_d)
		return date
	end

# 大会結果CSVファイルの内容を取得する
# ファイル内容がシフトJIS(CP932)の場合はUTF-8に変換する。
	def get_csvdata(csvfile = "")
		# csvfileがUTF-8またはShift_JIS(CP932)でない場合はエラー
		full_csvfile = @@yeardir + '/' +csvfile
		f = File.open(full_csvfile, "r:BOM|UTF-8")
		@@csv = f.read
		f.close
		enc = NKF.guess(@@csv).to_s
		if enc == "UTF-8"
		else
			f = File.open(full_csvfile, "r:CP932")
			@@csv = f.read
			f.close
			enc = NKF.guess(@@csv).to_s
			if enc == "Shift_JIS" || enc == "CP932" || enc == "Windows-31J"
				@@csv.encode!("UTF-8", "CP932")
			else
				puts "対戦記録表のデータに誤りがあります。"
				puts "対戦記録表のデータは「UTF-8」または「CP932」ではありません。(#{enc})"
				exit 1
			end
		end

		return @@csv
	end

# 大会結果CSVファイルを読み取り専用にする
	def readonly_taikaicsv(filename = "")
		full_csvfile = @@yeardir + '/' + filename
		FileUtils.chmod(0444, full_csvfile)
	end

#######################################################################
# @@tooldirにある一時ファイルの削除

# 存在確認後にファイル削除を行う（ツールのディレクトリのみ）
	def delete_file(filename = "")
		fullpath = @@tooldir + '/' + filename
		if FileTest.exist?(fullpath) == true
			File.delete(fullpath)
		end
	end

# 集計前に一時ファイルをまとめて削除する
	def delete_tempfile_before_syuukei
		self.delete_file FILE_ERROR
		self.delete_file FILE_ERROR_BAK
		self.delete_file FILE_MEIBO_BAK
		self.delete_file FILE_RANKING

		if DEBUG_MODE == false
			self.delete_file FILE_CSVLIST
		end

		# delete_tempfile_after_ranking と同内容
		self.delete_file FILE_GAME
		self.delete_file FILE_PLAYERS
		self.delete_file FILE_RESULT
		self.delete_file FILE_RESULT1
		self.delete_file FILE_RESULT2
		self.delete_file FILE_SAMENAME
		self.delete_file FILE_SAMENAME2
		self.delete_file FILE_SYUUKEI
		self.delete_file FILE_MERGE
		self.delete_file FILE_RANKFILE

		# delete_tempfile_after_hosei と同内容
		self.delete_file FILE_PT_MEIBO
		self.delete_file FILE_PT_GREP_GAME
	end

# ランキング作成後に一時ファイルをまとめて削除する
	def delete_tempfile_after_ranking
		if DEBUG_MODE == false
			self.delete_file FILE_GAME
			self.delete_file FILE_PLAYERS
			self.delete_file FILE_RESULT
			self.delete_file FILE_RESULT1
			self.delete_file FILE_RESULT2
			self.delete_file FILE_SAMENAME
			self.delete_file FILE_SAMENAME2
			self.delete_file FILE_SYUUKEI
			self.delete_file FILE_MERGE
			self.delete_file FILE_RANKFILE
		end
	end

# 持ち点初期値補正後に一時ファイルをまとめて削除する
	def delete_tempfile_after_hosei
		if DEBUG_MODE == false
			self.delete_file FILE_PT_MEIBO
			self.delete_file FILE_PT_GREP_GAME
		end
	end


#######################################################################
# 集計済み大会のYDBリストに関する処理

# 集計済み大会のYDBリストのフルパス名を得る
	def fullpath_ydb
		return @@ydbfile
	end

# 集計対象の西暦年のYDBリストを準備する
	def make_ydb
		if FileTest.exist?(@@ydbfile) == false
			fp = File.open(@@ydbfile, "w")
			fp.close
		end

		@@ydb = []
		ydbbuf = []
		i = 0

		File.open(@@ydbfile, "r") { |f|
			ydbbuf = f.read
		}

		ydbbuf.each_line { |f|
			csvfile = f.chomp
			@@ydb[i] = csvfile
			i = i + 1
		}
	end

# 集計済み大会としてYDBリストに登録されているか(1:有,0:無)
	def exist_taikai_in_ydb(taikai = "")
		@@ydb.each { |t|
			if t == taikai
				return 1
			end
		}
		return 0
	end

# 集計済み大会をYDBリストに追加登録する
	def add_taikai_to_ydb(taikai = "")
		File.open(@@ydbfile, "a") { |f|
			f.puts taikai
		}
	end

# YDBリストの最終登録大会のシステム整合性確認を行う
	def integrity_last_ydb
		# 指定された西暦年(年替わりの場合は前年)のYDBリストを参照する
		if FileTest.exist?(@@ydbfile) == true && FileTest.size(@@ydbfile) > 0
			ydbf = @@ydbfile
		elsif FileTest.exist?(@@ydbfile_ly) == true && FileTest.size(@@ydbfile_ly) > 0
			ydbf = @@ydbfile_ly
		else
			STDERR.puts "指定された西暦年のYDBファイルが見つかりません。"
			exit 1
		end

		# 最終登録大会の大会結果CSVファイル名を得る
		ydblist = []
		f = File.open(ydbf, "r")
		while s = f.gets
			ydblist.push s
		end
		f.close
		ydblist.sort!.reverse!
		csvfile = ydblist[0].chomp

		# 最終登録大会が各ディレクトリで一貫性のある状態になっているか確認する
		gamef  = @@gamedir + '/game-' + csvfile.sub(/\.[cC][sS][vV]$/, '.txt')
		meibof = @@meibodir + '/meibo-' + csvfile.sub(/\.[cC][sS][vV]$/, '.txt')
		rankf  = @@rankdir + '/' + csvfile.sub(/\.[cC][sS][vV]$/, '.d1')

		gamelist = Dir.glob(@@gamedir + "/game-*.txt")
		gamelist.sort!.reverse!
		meibolist = Dir.glob(@@meibodir + "/meibo-*.txt")
		meibolist.sort!.reverse!
		ranklist = Dir.glob(@@rankdir + "/*.d1")
		ranklist.sort!.reverse!

		integrity_result = csvfile.sub(/\.[cC][sS][vV]$/, '') + ':'
		errflag = 0
		if gamef == gamelist[0]
			integrity_result = integrity_result + " game:Pass"
		else
			integrity_result = integrity_result + " game:Fail"
			errflag = 1
		end

		if meibof == meibolist[0]
			integrity_result = integrity_result + " meibo:Pass"
		else
			integrity_result = integrity_result + " meibo:Fail"
			errflag = 1
		end

		if rankf == ranklist[0]
			integrity_result = integrity_result + " rank:Pass"
		else
			integrity_result = integrity_result + " rank:Fail"
			errflag = 1
		end

		if errflag == 1
			STDERR.puts "YDBリストの最終登録大会と整合性のないディレクトリがあります。"
			STDERR.puts integrity_result
			exit 1
		end
	end


#######################################################################
# ランクファイルに関する処理

# 大会結果csvfileからすべてのランクファイル名(d1,d2,d3)及び大会開催日を得る
	def all_rankfile_name(csvfile = "")
		meibo1 = @@rankdir + '/' + csvfile.sub(/\.[cC][sS][vV]$/, '.d1')
		meibo2 = @@rankdir + '/' + csvfile.sub(/\.[cC][sS][vV]$/, '.d2')
		meibo3 = @@rankdir + '/' + csvfile.sub(/\.[cC][sS][vV]$/, '.d3')
		date = self.date_ymd_taikaicsv(csvfile)
		return meibo1, meibo2, meibo3, date
	end

# 逆順の.d1ランクファイル一覧を得る
	def rankfile_d1_reverse_list
		d1 = []
		d1 = Dir.glob("*.d1", base: "#{@@rankdir}").sort!.reverse!
		return d1
	end

# ランクファイルのフルパス名を得る
	def fullpath_rankfile(filename="")
		return "#{@@rankdir}/#{filename}"
	end

#######################################################################
# コード化された順序付き対戦記録に関する処理

# コード化された順序付き対戦記録を保存する
	def save_game(csvfile = "")
		tname = @@gamedir + '/game-' + csvfile.sub(/\.[cC][sS][vV]$/, '.txt')
		FileUtils.copy(FILE_GAME, tname)
	end


#######################################################################
# 成績名簿マスターに関する処理

# 集計前の(手作業された可能性のある)成績名簿マスターを一時保存する
	def save_meibo
		FileUtils.copy(FILE_MEIBO, FILE_MEIBO_SAV)
	end

# 集計マージ後の成績名簿マスター候補を正式化する
	def baseline_meibo(csvfile = "")
		tname = @@meibodir + '/meibo-' + csvfile.sub(/\.[cC][sS][vV]$/, '.txt')
		FileUtils.copy(FILE_MERGE, FILE_MEIBO)
		FileUtils.copy(FILE_MERGE, FILE_MEIBO_BAK)
		FileUtils.copy(FILE_MERGE, tname)
	end

# 初期値補正した成績名簿マスター候補を正式化する
	def make_hosei_meibo(csvfile = "")
		tname = @@meibodir + '/meibo-' + csvfile.sub(/\.[cC][sS][vV]$/, '.txt')
		FileUtils.copy(FILE_PT_MEIBO, FILE_MEIBO)
		FileUtils.copy(FILE_PT_MEIBO, FILE_MEIBO_BAK)
		FileUtils.copy(FILE_PT_MEIBO, tname)
	end


#[End of Class] RatingFiles
end

